// BFFIterator
//------------------------------------------------------------------------------

// Includes
//------------------------------------------------------------------------------
#include "BFFIterator.h"
//#include <Tools/FBuild/FBuildCore/FLog.h>
//#include <Tools/FBuild/FBuildCore/Graph/NodeGraph.h>

//#include <Core/Tracing/Tracing.h>

//#include <stdarg.h>
//#include <stdio.h>

// SkipWhiteSpace
//------------------------------------------------------------------------------
void BFFIterator::SkipWhiteSpace()
{
	while ( !IsAtEnd() )
	{
		if ( IsAtWhitespace() )
		{
			(*this)++;
		}
		else
		{
			break;
		}
	}
}

// SkipComment
//------------------------------------------------------------------------------
void BFFIterator::SkipComment()
{
	// keep going until we hit the end of a line
	while ( !IsAtEnd() )
	{
		bool atLineEnd = (*m_Pos == '\n' );
		(*this)++;
		if ( atLineEnd )
		{
			break;
		}
	}
}

// SkipString
//------------------------------------------------------------------------------
void BFFIterator::SkipString( char quote )
{
	// start on open char (first ++ will handle this)
	ASSERT( *m_Pos == quote );

	while ( !IsAtEnd() )
	{
		m_Pos++;

		const char c = *m_Pos;

		// escape char?
		if ( c == '^' )
		{
			m_Pos++; // extra increment to skip escaped char
			continue;
		}

		// closing quote?
		if ( *m_Pos == quote )
		{
			return;
		}
	}
}

// SkipWhiteSpaceAndComments
//------------------------------------------------------------------------------
void BFFIterator::SkipWhiteSpaceAndComments()
{
	SkipWhiteSpace();
	while ( *m_Pos == ';' )
	{
		SkipComment();
		SkipWhiteSpace();
	}
}

// SkipVariableName
//------------------------------------------------------------------------------
void BFFIterator::SkipVariableName()
{
	while ( !IsAtEnd() )
	{
		if ( !IsAtValidVariableNameCharacter() )
		{
			return;
		}

		// character still part of variable name
		(*this)++;
	}
}

// SkipFunctionName
//------------------------------------------------------------------------------
void BFFIterator::SkipFunctionName()
{
	while ( !IsAtEnd() )
	{
		if ( IsAtValidFunctionNameCharacter() )
		{
			// character still part of variable name
			(*this)++;
			continue;
		}

		break; // hit the end of a non-valid character for a function name
	}
}

// ParseToNext
//------------------------------------------------------------------------------
bool BFFIterator::ParseToNext( char c )
{
	if ( IsAtEnd() )
	{
		return false;
	}

	char prev = '\000';
	do
	{
		(*this)++;
		if ( *m_Pos == c )
		{
			if ( prev != '^' )
			{
				return true;
			}
		}
		prev = *m_Pos;
	} while ( !IsAtEnd() );

	return false;
}

// ParseToMatchingBrace
//------------------------------------------------------------------------------
bool BFFIterator::ParseToMatchingBrace( char openBrace, char closeBrace )
{
	ASSERT( *m_Pos == openBrace );

	do
	{
		m_Pos++;

		// hit a nested brace?
		if ( *m_Pos == openBrace )
		{
			if ( ParseToMatchingBrace( openBrace, closeBrace ) == false )
			{
				return false;
			}
			continue;
		}

		// hit a string?
		if ( ( *m_Pos == '\'' ) || ( *m_Pos == '"' ) )
		{
			SkipString( *m_Pos );
		}

		// hit a comment?
		if ( *m_Pos == ';' )
		{
			SkipComment();
		}

		// hit the close brace?
		if ( *m_Pos == closeBrace )
		{
			return true;
		}

		// a regular charater.... keep searching

	} while ( !IsAtEnd() );

	return false;
}

// IsAtValidVariableNameCharacter
//------------------------------------------------------------------------------
bool BFFIterator::IsAtValidVariableNameCharacter() const
{
	const char c = *m_Pos;
	return ( ( ( c >= 'a' ) && ( c <= 'z' ) ) ||
			 ( ( c >= 'A' ) && ( c <= 'Z' ) ) ||
			 ( ( c >= '0' ) && ( c <= '9' ) ) ||
			 ( c == '_' ) );
}

// IsAtValidFunctionNameCharacter
//------------------------------------------------------------------------------
bool BFFIterator::IsAtValidFunctionNameCharacter() const
{
	const char c = *m_Pos;
	return ( ( ( c >= 'a' ) && ( c <= 'z' ) ) ||
			 ( ( c >= 'A' ) && ( c <= 'Z' ) ) ||
			 ( ( c >= '0' ) && ( c <= '9' ) ) ||
			 ( c == '_' ) );
}

// IsAtWhitespace
//------------------------------------------------------------------------------
bool BFFIterator::IsAtWhitespace() const
{
	char c = *m_Pos;
	return ( ( c == '\n' ) ||
			 ( c == '\r' ) ||
			 ( c == '\t' ) ||
			 ( c == ' ' ) );
}

// EmitParseError
//------------------------------------------------------------------------------
/*void BFFIterator::EmitParseError( const char * message, ... ) const
{
	ASSERT( message );

	const size_t BUFFER_SIZE( 4096 );
	char buffer[ BUFFER_SIZE ];

	va_list args;
	va_start(args, message);
	vsnprintf_s( buffer, BUFFER_SIZE, _TRUNCATE, message, args );
	va_end( args );

	// get human readable info about the position of the error
	uint32_t line = 0;
	uint32_t column = 0;
	const char * lineStart = nullptr;
	GetPosInfo( line, column, lineStart );

	// convert to full path and '/'->'\' cleanup
	const AStackString<> fileName( m_FileName );
	AStackString<> fullPath;
	NodeGraph::CleanPath( fileName, fullPath );

	// deliberately using OUTPUT here to avoid "Error:" in front
	OUTPUT( "%s(%u):(%u) BFF Parse Error - %s\n", fullPath.Get(), line, column, buffer );

	// find the line end
	BFFIterator lineEnd( *this );
	while ( !lineEnd.IsAtEnd() )
	{
		if (( *lineEnd != '\r' ) && ( *lineEnd != '\n' ))
		{
			lineEnd++;
			continue;
		}
		break;
	}

	// if line is too crazy to be useful, don't print anything more
	size_t lineLength = lineEnd.GetCurrent() - lineStart;
	if ( lineLength >= 256 )
	{
		return;
	}

	// print the problematic line
	AString::Copy( lineStart, buffer, lineLength );
	FLOG_ERROR( "%s", buffer );

	// point to the specific pos where parsing broke
	// (taking into account tabs)
	char * c = buffer;
	const char * end = c + column - 1;
	for ( ; c < end; ++c )
	{
		if ( *c != '\t' )
		{
			*c = ' ';
		}
	}

	AString::Copy( "^", c, 1 );
	FLOG_ERROR( buffer );
	AString::Copy( "\\--here", c, 8 );
	FLOG_ERROR( buffer );
}
*/

// GetPosInfo
//------------------------------------------------------------------------------
void BFFIterator::GetPosInfo( uint32_t & line,
							  uint32_t & column,
							  const char * & lineStart ) const
{
	// count the number of lines from the start of the file to
	// the current iterator position
	const char * p = m_MinPos;
	lineStart = m_MinPos;
	line = 1;
	while ( p < m_Pos ) // loop to current iterator pos
	{
		bool atLineEnd = ( *p == '\n' );
		p++;
		if ( atLineEnd )
		{
			lineStart = p;
			line++;
		}
	}
	column = (uint32_t)( ( p - lineStart ) + 1 );
}

//------------------------------------------------------------------------------
